/*
 *   file: form_card.c
 *  
 *  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <libral.h>

#include "app.h"
#include "form.h"
#include "form_card.h"
#include "app.h"
#include "groups_cb.h"
#include "models.h"
#include "types.h"
#include "utils.h"


struct _RubricaFormCardPrivate {
  RubricaGroupSet* set;

  GtkWidget* card;
  GtkWidget* group_tree;
  GtkWidget* del;
  GtkWidget* rank;  
  
  gboolean dispose_has_run;
};


static void rubrica_form_card_class_init (RubricaFormCardClass* klass);
static void rubrica_form_card_init       (RubricaFormCard* obj);

static void rubrica_form_card_dispose    (RubricaFormCard* obj);
static void rubrica_form_card_finalize   (RubricaFormCard* obj);

static gboolean extract_groups (GtkTreeModel *model, GtkTreePath *path,
				GtkTreeIter *iter, gpointer data); 

static void form_card_extract_data (RubricaFormCard* form, RubricaCard *card);
static void form_card_display_data (RubricaFormCard* form, RubricaCard *card);
static void form_card_clean_form   (RubricaFormCard* form);


static RubricaFormClass* parent_class;


GType
rubrica_form_card_get_type()
{
  static GType form_card_type = 0;
  
  if (!form_card_type)
    {
      static const GTypeInfo form_card_info =
	{
	  sizeof(RubricaFormCardClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_form_card_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaFormCard),
	  0,
	  (GInstanceInitFunc) rubrica_form_card_init
	};

      form_card_type = g_type_register_static (RUBRICA_FORM_TYPE, 
					       "RubricaFormCard",
					       &form_card_info, 0);
    }
  
  return form_card_type;
}


static void
rubrica_form_card_class_init(RubricaFormCardClass* klass)
{
  GObjectClass *class;
 
  class  = G_OBJECT_CLASS (klass);
  class->dispose  = (GObjectFinalizeFunc) rubrica_form_card_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_form_card_finalize;

  parent_class = RUBRICA_FORM_CLASS(klass);
  parent_class->extract_data = (gpointer) form_card_extract_data;
  parent_class->display_data = (gpointer) form_card_display_data;
  parent_class->clean_form   = (gpointer) form_card_clean_form;
}


static void
rubrica_form_card_init(RubricaFormCard* self)
{
  self->private = g_new(RubricaFormCardPrivate, 1);
  
  self->private->set        = NULL;
  self->private->card       = NULL;
  self->private->group_tree = NULL;
  self->private->del        = NULL;
  self->private->rank       = NULL;

  self->private->dispose_has_run = FALSE;
}

 

static void 
rubrica_form_card_dispose (RubricaFormCard* self)
{
  g_return_if_fail(IS_RUBRICA_FORM(self));
  
  if (self->private->dispose_has_run)
    return;
  
  g_object_unref(self->private->card);
  g_object_unref(self->private->group_tree);
  g_object_unref(self->private->del);
  g_object_unref(self->private->rank);
  
  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_form_card_finalize (RubricaFormCard* self)
{
  g_return_if_fail(IS_RUBRICA_FORM(self));
  
  g_free(self->private);
  self->private = NULL;
}

static gboolean 
extract_groups (GtkTreeModel *model, GtkTreePath *path,
		GtkTreeIter *iter, gpointer data)
{
  RubricaCard* card = (RubricaCard*) data;
  gchar* name = NULL;
  gchar* owner = NULL;
  gchar* uri = NULL;
  gboolean belong = FALSE;

  gtk_tree_model_get(model, iter, 
		     GROUP_BELONG_COLUMN, &belong,
		     GROUP_LABEL_COLUMN,  &name,
		     GROUP_OWNER_COLUMN,  &owner,
		     GROUP_PIXMAP_PATH,   &uri,
		     -1);
  if (belong) 
    {
      RubricaGroup* group;

      group = rubrica_group_new();
      g_object_set(group, 
		   "group-name",   name, 
		   "group-owner",  owner,
		   "group-pixmap", uri, 
		   "enabled",      TRUE, 
		   NULL);

      rubrica_card_add_group(RUBRICA_CARD(card), group);

      belong = FALSE;
    }
  
  return FALSE;  
}



static void 
form_card_extract_data (RubricaFormCard* form, RubricaCard *card)
{
  GtkTreeModel* model = NULL;
  const gchar *name;
  gboolean dele;
  gint rank;
  
  name = gtk_entry_get_text(GTK_ENTRY(form->private->card));
  dele = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->private->del));
  rank = gtk_combo_box_get_active(GTK_COMBO_BOX(form->private->rank));

  g_object_set(RUBRICA_CARD(card), 
	       "deletable", dele, "deleted", FALSE, "marked", FALSE, 
	       "card-name", name, "rank", rank, NULL);  
  
  /* extract groups's data */
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->private->group_tree));
  if (model)
    {
      gtk_tree_model_foreach(GTK_TREE_MODEL(model), extract_groups, card);
      g_object_unref(model);
    }  
}


static void 
form_card_display_data (RubricaFormCard* form, RubricaCard *card)
{
  GtkTreeView* tree;
  gpointer data;
  gchar* name;
  gboolean deletable;
  RubricaRank rank;
  
  tree = GTK_TREE_VIEW(form->private->group_tree);  
  
  g_object_get(RUBRICA_CARD(card), 
	       "card-name", &name, 
	       "deletable", &deletable, 
	       "rank",      &rank,
	       NULL);

  gtk_entry_set_text(GTK_ENTRY(form->private->card), name);  
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->private->del),
			       deletable);
  gtk_combo_box_set_active(GTK_COMBO_BOX(form->private->rank), rank); 
  
  rubrica_group_set_disable_all (form->private->set);
  data = rubrica_card_get_group(RUBRICA_CARD(card));
  for (; data; data = rubrica_card_get_next_group(RUBRICA_CARD(card)))
    rubrica_group_set_enable_group(form->private->set, data);

  rubrica_model_redisplay_groups_view(GTK_TREE_VIEW(tree), form->private->set);
}


static void 
form_card_clean_form (RubricaFormCard* form)
{
  GtkTreeView* tree;
  
  tree = GTK_TREE_VIEW(form->private->group_tree);  

  gtk_entry_set_text(GTK_ENTRY(form->private->card), "");  
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->private->del), FALSE);
  gtk_combo_box_set_active(GTK_COMBO_BOX(form->private->rank), 
			   RUBRICA_RANK_NORMAL); 
  
  rubrica_group_set_disable_all (form->private->set);
  rubrica_model_redisplay_groups_view(GTK_TREE_VIEW(tree), form->private->set);
}


/*   ***************************** Public *****************************
*/


/**
 * rubrica_form_card_new
 *
 * create a #RubricaFormCard
 *
 * Returns: a new #RubricaFormCard
 */
RubricaFormCard*
rubrica_form_card_new(RubricaApp* app, GladeXML *gui)
{
  RubricaFormCard* form;
  GtkTreeView      *tree;
  GtkTreeModel     *model;
  GtkTreeSelection *selection;

  form = g_object_new(rubrica_form_card_get_type(), NULL);
  
  form->private->card       = glade_xml_get_widget(gui, "card");
  form->private->group_tree = glade_xml_get_widget(gui, "group_tree");
  form->private->del        = glade_xml_get_widget(gui, "check_del");
  form->private->rank       = glade_xml_get_widget(gui, "rank");  
  form->private->set        = rubrica_app_get_groups(app);

  gtk_combo_box_set_active(GTK_COMBO_BOX(form->private->rank),
			   RUBRICA_RANK_NORMAL); 
    
  tree  = (GtkTreeView*) form->private->group_tree;
  model = rubrica_model_new(GROUP_MODEL);
  gtk_tree_view_set_model(GTK_TREE_VIEW(tree), GTK_TREE_MODEL(model));
    
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
  gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);

  rubrica_model_add_columns(GTK_TREE_VIEW(tree), GROUP_COLUMNS); 
  rubrica_model_populate_groups_view(tree, form->private->set);  

  g_signal_connect(G_OBJECT(form->private->set), "group_added", 
		   G_CALLBACK(on_rubrica_add_to_groupstree), tree);
  
  g_signal_connect(G_OBJECT(form->private->set), "group_removed", 
		   G_CALLBACK(on_rubrica_remove_from_groupstree), tree);
  
  g_signal_connect(GTK_OBJECT (tree), "button_press_event",
  		   G_CALLBACK (on_rubrica_form_groups_event), app);
  
  g_object_unref(model); 
  
  return form;
}


/**
 * rubrica_form_card_free
 * @form: a #RubricaFormCard
 * 
 * free the #RubricaFormCard
 */
void
rubrica_form_card_free(RubricaFormCard* form)
{
  g_return_if_fail(IS_RUBRICA_FORM(form));

  g_object_unref(form);   
}
