/*
 *   file: form_notes.c
 *  
 *  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <libral.h>

#include "app.h"
#include "form.h"
#include "form_notes.h"
#include "app.h"
#include "groups_cb.h"
#include "models.h"
#include "types.h"
#include "utils.h"


struct _RubricaFormNotesPrivate {
  GtkWidget* has_partner;     /* */
  GtkWidget* table;   
  GtkWidget* partner;
  GtkWidget* sbirth;           /* birthday's partner entry */
  GtkWidget* anniversary;      /* anniversary entry       */  
  GtkWidget* partner_bt;       /* calendar button for partner birthday */
  GtkWidget* anniversary_bt;   /* calendar button for anniversary */
  GtkWidget* notes;
  GtkWidget* pubkey;
  
  gboolean dispose_has_run;
};


static void rubrica_form_notes_class_init (RubricaFormNotesClass* klass);
static void rubrica_form_notes_init       (RubricaFormNotes* obj);

static void rubrica_form_notes_dispose    (RubricaFormNotes* obj);
static void rubrica_form_notes_finalize   (RubricaFormNotes* obj);


static gchar* extract_text            (GtkWidget *text);

static void   form_notes_extract_data (RubricaFormNotes* form, 
				       RubricaCard *card);
static void   form_notes_display_data (RubricaFormNotes* form,
				       RubricaCard *card);
static void   form_notes_clean_form   (RubricaFormNotes* form);


static RubricaFormClass* parent_class;


GType
rubrica_form_notes_get_type()
{
  static GType form_notes_type = 0;
  
  if (!form_notes_type)
    {
      static const GTypeInfo form_notes_info =
	{
	  sizeof(RubricaFormNotesClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_form_notes_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaFormNotes),
	  0,
	  (GInstanceInitFunc) rubrica_form_notes_init
	};

      form_notes_type = g_type_register_static (RUBRICA_FORM_TYPE, 
					       "RubricaFormNotes",
					       &form_notes_info, 0);
    }
  
  return form_notes_type;
}


static void
rubrica_form_notes_class_init(RubricaFormNotesClass* klass)
{
  GObjectClass *class;
 
  class  = G_OBJECT_CLASS (klass);
  class->dispose  = (GObjectFinalizeFunc) rubrica_form_notes_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_form_notes_finalize;

  parent_class = RUBRICA_FORM_CLASS(klass);
  parent_class->extract_data = (gpointer) form_notes_extract_data;
  parent_class->display_data = (gpointer) form_notes_display_data;
  parent_class->clean_form   = (gpointer) form_notes_clean_form;
}


static void
rubrica_form_notes_init(RubricaFormNotes* self)
{
  self->private = g_new(RubricaFormNotesPrivate, 1);

  self->private->has_partner    = NULL;
  self->private->table          = NULL;   // just a container
  self->private->partner        = NULL;
  self->private->sbirth         = NULL;
  self->private->anniversary    = NULL;
  self->private->partner_bt     = NULL;
  self->private->anniversary_bt = NULL;
  self->private->notes          = NULL;
  self->private->pubkey         = NULL;

  self->private->dispose_has_run = FALSE;
}

 

static void 
rubrica_form_notes_dispose (RubricaFormNotes* self)
{
  g_return_if_fail(IS_RUBRICA_FORM(self));
  
  if (self->private->dispose_has_run)
    return;

  g_object_unref(self->private->has_partner);
  g_object_unref(self->private->table);
  g_object_unref(self->private->partner);
  g_object_unref(self->private->sbirth);
  g_object_unref(self->private->anniversary);
  g_object_unref(self->private->partner_bt);
  g_object_unref(self->private->anniversary_bt);
  g_object_unref(self->private->notes);
  g_object_unref(self->private->pubkey);
  
  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_form_notes_finalize (RubricaFormNotes* self)
{
  g_return_if_fail(IS_RUBRICA_FORM(self));
  
  g_free(self->private);
  self->private = NULL;
}


/** 
 * extract_text
 * @text: the text widget pointer 
 *  
 * extract text stored into the text widget.  
 * 
 * return: an allocated UTF-8 string  
 */ 
gchar*
extract_text (GtkWidget *text) 
{
  GtkTextIter start, end;
  GtkTextBuffer *text_buffer;
  gchar *ret_val;                /* an allocated UTF-8 string */
  
  text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (text));
  
  gtk_text_buffer_get_bounds(text_buffer, &start, &end);
  ret_val = gtk_text_buffer_get_text(text_buffer, &start, &end, FALSE);

  return ret_val;
}



static void 
form_notes_extract_data (RubricaFormNotes* form, RubricaCard *card)
{
  RubricaNotes* notes;
  gboolean has_partner = FALSE;
  const gchar *partner;
  const gchar *date = NULL;

  g_return_if_fail(IS_RUBRICA_CARD(card));

  notes = rubrica_notes_new();

  has_partner = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
					     (form->private->has_partner));

  if (has_partner)
    {
      partner = gtk_entry_get_text(GTK_ENTRY(form->private->partner));
      
      date = gtk_entry_get_text(GTK_ENTRY(form->private->sbirth));
      if (g_ascii_strcasecmp(date, "unknown") != 0)
	{
	  GtkEntry* birth = (GtkEntry*) form->private->sbirth;
	  gint day = -1, month = -1, year = -1;
	  
	  day   = (gint) g_object_get_data(G_OBJECT(birth), "day");
	  month = (gint) g_object_get_data(G_OBJECT(birth), "month") + 1;
	  year  = (gint) g_object_get_data(G_OBJECT(birth), "year");
	  
	  rubrica_notes_set_know_birthday(notes, TRUE);
	  rubrica_notes_set_birthday(notes, day, month, year);
	}
      
      date = gtk_entry_get_text(GTK_ENTRY(form->private->anniversary));
      if (g_ascii_strcasecmp(date, "unknown") != 0)
	{
	  GtkEntry* ann = (GtkEntry*) form->private->anniversary;
	  gint day = -1, month = -1, year = -1;
	  
	  day   = (gint) g_object_get_data(G_OBJECT(ann), "day");
	  month = (gint) g_object_get_data(G_OBJECT(ann), "month") + 1;
	  year  = (gint) g_object_get_data(G_OBJECT(ann), "year");
	  
	  rubrica_notes_set_know_anniversary(notes, TRUE);
	  rubrica_notes_set_anniversary(notes, day, month, year);
	}
    }
  else
    {
      partner = "";
      rubrica_notes_set_know_birthday(notes, FALSE);
      rubrica_notes_set_birthday(notes, -1, -1, -1);
      rubrica_notes_set_know_anniversary(notes, FALSE);
      rubrica_notes_set_anniversary(notes, -1, -1, -1);
    }
  
  g_object_set(notes, 
	       "has-partner",  has_partner, 
	       "partner-name", partner,
	       "children",     "", 
	       "other-notes",  extract_text(form->private->notes),
	       "pubkey",       extract_text(form->private->pubkey), 
	       NULL);  
  
  rubrica_personal_card_set_notes(RUBRICA_PERSONAL_CARD(card), notes);
}


static void 
form_notes_display_data (RubricaFormNotes* form, RubricaCard *card)
{
  RubricaNotes* notes;
  GtkTextView* view;
  GtkTextIter start, end;
  GtkTextBuffer *buffer;
  gchar *pubkey, *other, *partner;
  gboolean has_partner;

  notes = rubrica_personal_card_get_notes (RUBRICA_PERSONAL_CARD(card));
  
  g_object_get(notes, "pubkey", &pubkey, "other-notes", &other, 
	       "has-partner", &has_partner, "partner-name", &partner, NULL);
  
  /*    Public Key
  */
  view   = GTK_TEXT_VIEW(form->private->pubkey);
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
  gtk_text_buffer_get_bounds(buffer, &start, &end);
  gtk_text_buffer_delete(buffer, &start, &end);
  gtk_text_buffer_get_iter_at_offset(buffer, &start, 0);  
  gtk_text_buffer_insert (buffer, &start, pubkey, -1);
  
  /*    Other notes
  */
  view   = GTK_TEXT_VIEW(form->private->notes);
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
  gtk_text_buffer_get_bounds(buffer, &start, &end);
  gtk_text_buffer_delete(buffer, &start, &end);
  gtk_text_buffer_get_iter_at_offset(buffer, &start, 0);  
  gtk_text_buffer_insert (buffer, &start, other, -1);

  if (has_partner)
    {
      gint day, month, year;
      gchar* date;
      
      gtk_widget_set_sensitive(GTK_WIDGET(form->private->table), TRUE); 
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->private->has_partner),
				   TRUE);
      gtk_entry_set_text(GTK_ENTRY(form->private->partner), partner);

      if (rubrica_notes_know_birthday(notes))
	{
	  date  = rubrica_notes_get_birthdate(notes);
	  day   = atoi(rubrica_notes_get_birth_day(notes));
	  month = atoi(rubrica_notes_get_birth_month(notes)) - 1;
	  year  = atoi(rubrica_notes_get_birth_year(notes));
	  
	  gtk_entry_set_text(GTK_ENTRY(form->private->sbirth), date);

	  g_object_set_data(G_OBJECT(form->private->sbirth), 
			    "day", GINT_TO_POINTER(day));
	  g_object_set_data(G_OBJECT(form->private->sbirth), 
			    "month", GINT_TO_POINTER(month));
	  g_object_set_data(G_OBJECT(form->private->sbirth), 
			    "year", GINT_TO_POINTER(year));	  
	}
      
      if (rubrica_notes_know_anniversary(notes))
	{
	  gchar *day, *month, *year;
	  gchar* date;

	  date  = rubrica_notes_get_anniversarydate(notes);
	  day   = rubrica_notes_get_anniversary_day(notes); 
	  month = rubrica_notes_get_anniversary_month(notes);  
	  year  = rubrica_notes_get_anniversary_year(notes);
	  
	  gtk_entry_set_text(GTK_ENTRY(form->private->anniversary), date);

	  g_object_set_data(G_OBJECT(form->private->anniversary), 
			    "day", GINT_TO_POINTER(day));
	  g_object_set_data(G_OBJECT(form->private->anniversary), 
			    "month",GINT_TO_POINTER(month));
	  g_object_set_data(G_OBJECT(form->private->anniversary), 
			    "year", GINT_TO_POINTER(year));	  
	}        
    }
}


static void 
form_notes_clean_form (RubricaFormNotes* form)
{
  // TODO
}





/*    Callbacks
*/

void 
rubrica_form_married_check_toggled(GtkToggleButton *togglebutton, 
				   gpointer data)
{
  GtkWidget* table = (GtkWidget*) data;
  gboolean married;
  
  married = RUBRICA_GET_CHECK(togglebutton);
  gtk_widget_set_sensitive(GTK_WIDGET(table), married);
}



/*   ***************************** Public *****************************
*/


/**
 * rubrica_form_notes_new
 *
 * create a #RubricaFormNotes
 *
 * Returns: a new #RubricaFormNotes
 */
RubricaFormNotes*
rubrica_form_notes_new(RubricaApp* app, GladeXML *gui)
{
  RubricaFormNotes* form;

  form = g_object_new(rubrica_form_notes_get_type(), NULL);
  
  form->private->has_partner    = glade_xml_get_widget(gui, "spouse_check");
  form->private->table          = glade_xml_get_widget(gui, "marriage_table");
  form->private->partner        = glade_xml_get_widget(gui, "spouse");
  form->private->sbirth         = glade_xml_get_widget(gui, "sbirth");
  form->private->partner_bt     = glade_xml_get_widget(gui, "spouse_bt");
  form->private->anniversary_bt = glade_xml_get_widget(gui, "anniversary_bt");
  form->private->anniversary    = glade_xml_get_widget(gui, "anniversary");
  form->private->notes          = glade_xml_get_widget(gui, "notes");
  form->private->pubkey         = glade_xml_get_widget(gui, "pubkey");

  gtk_widget_set_sensitive(GTK_WIDGET(form->private->table), FALSE);

  /*     notes checkbutton 
   */
  g_signal_connect(G_OBJECT(form->private->has_partner), "toggled", 
		   G_CALLBACK(rubrica_form_married_check_toggled), 
		   form->private->table);

  g_signal_connect (G_OBJECT (form->private->partner_bt), "clicked", 
		    G_CALLBACK (rubrica_utils_calendar_ui), 
		    form->private->sbirth);  

  g_signal_connect (G_OBJECT (form->private->anniversary_bt), "clicked", 
		    G_CALLBACK (rubrica_utils_calendar_ui), 
		    form->private->anniversary);
  
  return form;
}


/**
 * rubrica_form_notes_free
 * @form: a #RubricaFormNotes
 * 
 * free the #RubricaFormNotes
 */
void
rubrica_form_notes_free(RubricaFormNotes* form)
{
  g_return_if_fail(IS_RUBRICA_FORM(form));

  g_object_unref(form);   
}
