/*
 *   file: form_phone.c
 *  
 *  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <libral.h>

#include "app.h"
#include "form.h"
#include "form_phone.h"
#include "app.h"
#include "groups_cb.h"
#include "models.h"
#include "types.h"
#include "utils.h"


struct _RubricaFormPhonePrivate {
  RubricaGroupSet* set;

  GtkWidget* tel;
  GtkWidget* tree;
  GtkWidget* add;
  GtkWidget* del;
  GtkWidget* mod;
  GtkWidget* home;
  GtkWidget* work;
  GtkWidget* fax;
  GtkWidget* cell;
  GtkWidget* other;
  
  gboolean dispose_has_run;
};


static void rubrica_form_phone_class_init (RubricaFormPhoneClass* klass);
static void rubrica_form_phone_init       (RubricaFormPhone* obj);

static void rubrica_form_phone_dispose    (RubricaFormPhone* obj);
static void rubrica_form_phone_finalize   (RubricaFormPhone* obj);


static gboolean extract_telephones (GtkTreeModel *model, GtkTreePath *path,
			     GtkTreeIter *iter, gpointer data);


static void form_phone_extract_data (RubricaFormPhone* form, 
				     RubricaCard *card);
static void form_phone_display_data (RubricaFormPhone* form, 
				     RubricaCard *card);
static void form_phone_clean_form   (RubricaFormPhone* form);


static RubricaFormClass* parent_class;


GType
rubrica_form_phone_get_type()
{
  static GType form_phone_type = 0;
  
  if (!form_phone_type)
    {
      static const GTypeInfo form_phone_info =
	{
	  sizeof(RubricaFormPhoneClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_form_phone_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaFormPhone),
	  0,
	  (GInstanceInitFunc) rubrica_form_phone_init
	};

      form_phone_type = g_type_register_static (RUBRICA_FORM_TYPE, 
					       "RubricaFormPhone",
					       &form_phone_info, 0);
    }
  
  return form_phone_type;
}


static void
rubrica_form_phone_class_init(RubricaFormPhoneClass* klass)
{
  GObjectClass *class;
 
  class  = G_OBJECT_CLASS (klass);
  class->dispose  = (GObjectFinalizeFunc) rubrica_form_phone_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_form_phone_finalize;

  parent_class = RUBRICA_FORM_CLASS(klass);
  parent_class->extract_data = (gpointer) form_phone_extract_data;
  parent_class->display_data = (gpointer) form_phone_display_data;
  parent_class->clean_form   = (gpointer) form_phone_clean_form;
}


static void
rubrica_form_phone_init(RubricaFormPhone* self)
{
  self->private = g_new(RubricaFormPhonePrivate, 1);
  
  self->private->tel   = NULL;
  self->private->tree  = NULL;
  self->private->add   = NULL;
  self->private->mod   = NULL;
  self->private->del   = NULL;
  self->private->home  = NULL;
  self->private->work  = NULL;
  self->private->fax   = NULL;
  self->private->cell  = NULL;
  self->private->other = NULL;

  self->private->dispose_has_run = FALSE;
}

 

static void 
rubrica_form_phone_dispose (RubricaFormPhone* self)
{
  g_return_if_fail(IS_RUBRICA_FORM(self));
  
  if (self->private->dispose_has_run)
    return;
  
  g_object_unref(self->private->tel);
  g_object_unref(self->private->tree);
  g_object_unref(self->private->add);
  g_object_unref(self->private->mod);
  g_object_unref(self->private->del);
  g_object_unref(self->private->home);
  g_object_unref(self->private->work);
  g_object_unref(self->private->fax);
  g_object_unref(self->private->cell);
  g_object_unref(self->private->other);
  
  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_form_phone_finalize (RubricaFormPhone* self)
{
  g_return_if_fail(IS_RUBRICA_FORM(self));
  
  g_free(self->private->tel);
  g_free(self->private->tree);
  g_free(self->private->add);
  g_free(self->private->mod);
  g_free(self->private->del);
  g_free(self->private->home);
  g_free(self->private->work);
  g_free(self->private->fax);
  g_free(self->private->cell);
  g_free(self->private->other);

  self->private = NULL;
}



gboolean 
extract_telephones (GtkTreeModel *model, GtkTreePath *path,
		    GtkTreeIter *iter, gpointer data)
{
  RubricaCard* card = (RubricaCard*) data;
  RubricaTelephone* tel = NULL;
  RubricaTelephoneType ttype;
  gchar *number, *type;

  gtk_tree_model_get(model, iter, 
		     TELEPHONE_NUMBER_COLUMN, &number,
		     TELEPHONE_TYPE_COLUMN, &type,
		     -1);
  ttype = rubrica_telephone_encode_type(type);

  tel = rubrica_telephone_new();
  g_object_set(tel, "telephone-number", number, "telephone-type", ttype, NULL);
  rubrica_card_add_telephone(RUBRICA_CARD(card), tel);

  return FALSE;
}


static void 
form_phone_extract_data (RubricaFormPhone* form, RubricaCard *card)
{
  GtkTreeModel* model = NULL;

  g_return_if_fail(IS_RUBRICA_CARD(card));

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->private->tree));
  if (model)
    {
      gtk_tree_model_foreach(GTK_TREE_MODEL(model), extract_telephones, card);
      g_object_unref(model);
    }
}


static void 
form_phone_display_data (RubricaFormPhone* form, RubricaCard *card)
{
  GtkTreeView* tree;
  GtkTreeModel *model;
  GtkTreeIter iter;
  gpointer data;

  /*   telephone model/view 
   */
  tree  = (GtkTreeView*) form->private->tree;
  model = gtk_tree_view_get_model(tree);

  data = rubrica_card_get_telephone(RUBRICA_CARD(card));
  for (; data; data = rubrica_card_get_next_telephone(RUBRICA_CARD(card)))
    {
      RubricaTelephoneType ttype;
      gchar *number, *type;
      
      g_object_get(RUBRICA_TELEPHONE(data), "telephone-number", &number,
		   "telephone-type", &ttype, NULL);      
      type = rubrica_telephone_decode_type(ttype);
      
      gtk_list_store_append(GTK_LIST_STORE(model), &iter);
      gtk_list_store_set(GTK_LIST_STORE(model),   &iter, 
			 TELEPHONE_NUMBER_COLUMN, number,
			 TELEPHONE_TYPE_COLUMN,   type,
			 -1);
    }
}


static void 
form_phone_clean_form   (RubricaFormPhone* form)
{

}


/*  Callbacks
*/
void 
form_telephone_list_activated(GtkTreeSelection *selection, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  RubricaFormPhone *form = (RubricaFormPhone *) data;
  gchar *numb_str;
  gchar *type_str;
  gint type;

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->private->tree));

  if (gtk_tree_selection_get_selected(selection, &model, &iter))
    {
      gtk_tree_model_get(model, &iter, 
			 TELEPHONE_NUMBER_COLUMN, &numb_str, 
			 TELEPHONE_TYPE_COLUMN, &type_str,
			 -1);
      
      gtk_entry_set_text(GTK_ENTRY(form->private->tel), numb_str);
      type = rubrica_telephone_encode_type(type_str);

      switch (type)
	{
	case RUBRICA_TELEPHONE_HOME:
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->private->home),
				       TRUE);
	  break;
	case RUBRICA_TELEPHONE_WORK:
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->private->work),
				       TRUE);
	  break;
	case RUBRICA_TELEPHONE_FAX:
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->private->fax),
				       TRUE);
	  break;
	case RUBRICA_TELEPHONE_CELLPHONE:
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->private->cell),
				       TRUE);
	  break; 
	case RUBRICA_TELEPHONE_OTHER:
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->private->other),
				       TRUE);
	  break; 

	default:
	  break;
	}      
      
      gtk_widget_set_sensitive(form->private->mod, TRUE);
      gtk_widget_set_sensitive(form->private->del, TRUE);
      g_free(numb_str);
      g_free(type_str);
    }
}


void
form_tel_add_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  gchar *type_str = NULL;
  RubricaFormPhone *form = (RubricaFormPhone *) data;
  RubricaTelephone* tel;
  RubricaTelephoneType type;
  gchar* number;

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(form->private->tree));

  tel = rubrica_telephone_new();
  number = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->private->tel)));
  
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->private->home)))
    {
      type_str = "home";
      type = RUBRICA_TELEPHONE_HOME;
    }
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->private->work)))
    {  
      type_str = "work";
      type = RUBRICA_TELEPHONE_WORK;
    }  
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->private->fax)))
    {
      type_str = "fax";
      type = RUBRICA_TELEPHONE_FAX;
    }
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->private->cell)))
   {
     type_str = "cellphone";
     type = RUBRICA_TELEPHONE_CELLPHONE;
   }
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->private->other)))
    {
      type_str = "other";
      type = RUBRICA_TELEPHONE_OTHER;
    }

  g_object_set(tel, "telephone-number", number, "telephone-type", type, NULL);

  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
		     TELEPHONE_NUMBER_COLUMN, number,
		     TELEPHONE_TYPE_COLUMN, type_str,
		     -1);

  gtk_entry_set_text(GTK_ENTRY(form->private->tel), "");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->private->home), TRUE);
    
  gtk_widget_set_sensitive(GTK_WIDGET(form->private->mod), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(form->private->del), FALSE);  
}


void
form_tel_mod_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkTreeSelection *selection;
  RubricaFormPhone *form = (RubricaFormPhone *) data;
  RubricaTelephone* tel;
  RubricaTelephoneType type;
  gchar* number;
  gchar *type_str = NULL;

  model     = gtk_tree_view_get_model(GTK_TREE_VIEW(form->private->tree));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(form->private->tree));

  tel = rubrica_telephone_new();
  number = g_strdup(gtk_entry_get_text(GTK_ENTRY(form->private->tel)));

  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->private->home)))
    {
      type_str = "home";
      type = RUBRICA_TELEPHONE_HOME;
    }
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->private->work)))
    {
      type_str = "work";
      type = RUBRICA_TELEPHONE_WORK;
    }
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->private->fax)))
    {
      type_str = "fax";
      type = RUBRICA_TELEPHONE_FAX;
   }
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->private->cell)))
    {
      type_str = "cellphone";
      type = RUBRICA_TELEPHONE_CELLPHONE;
    }
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(form->private->other)))
    {
      type = RUBRICA_TELEPHONE_OTHER;
      type_str = "other";
    }

  g_object_set(tel, "telephone-number", number, "telephone-type", type, NULL);

  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
  {
    gint i;
    GtkTreePath *path;
    
    path = gtk_tree_model_get_path(model, &iter);
    i    = gtk_tree_path_get_indices(path)[0];
    
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
    gtk_tree_path_free(path);
    
    gtk_list_store_insert(GTK_LIST_STORE(model), &iter, i);
    gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
		       TELEPHONE_NUMBER_COLUMN, number, 
		       TELEPHONE_TYPE_COLUMN, type_str,
		       -1);

    gtk_entry_set_text(GTK_ENTRY(form->private->tel),"");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->private->home), TRUE);

    gtk_widget_set_sensitive(GTK_WIDGET(form->private->mod), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(form->private->del), FALSE);   
  }  
}


void
form_tel_del_clicked (GtkButton *button, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkTreeSelection *selection;
  RubricaFormPhone *form = (RubricaFormPhone *) data;

  model     = gtk_tree_view_get_model(GTK_TREE_VIEW(form->private->tree));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(form->private->tree));

  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
  {
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
    
    gtk_entry_set_text(GTK_ENTRY(form->private->tel),"");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(form->private->home), TRUE);

    gtk_widget_set_sensitive(GTK_WIDGET(form->private->mod), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(form->private->del), FALSE);   
  }  
}









/*   ***************************** Public *****************************
*/




/**
 * rubrica_form_phone_new
 *
 * create a #RubricaFormPhone
 *
 * Returns: a new #RubricaFormPhone
 */
RubricaFormPhone*
rubrica_form_phone_new(RubricaApp* app, GladeXML *gui)
{
  RubricaFormPhone* form;
  GtkTreeModel     *model;
  GtkTreeSelection *selection;

  form = g_object_new(rubrica_form_phone_get_type(), NULL);
  
  form->private->tel   = glade_xml_get_widget(gui, "tel");
  form->private->tree  = glade_xml_get_widget(gui, "tel_tree");
  form->private->add   = glade_xml_get_widget(gui, "tel_add");
  form->private->del   = glade_xml_get_widget(gui, "tel_del");
  form->private->mod   = glade_xml_get_widget(gui, "tel_mod");
  form->private->home  = glade_xml_get_widget(gui, "home");
  form->private->work  = glade_xml_get_widget(gui, "work");
  form->private->fax   = glade_xml_get_widget(gui, "fax");
  form->private->cell  = glade_xml_get_widget(gui, "cell");
  form->private->other = glade_xml_get_widget(gui, "other");
  
  gtk_widget_set_sensitive(GTK_WIDGET(form->private->mod), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(form->private->del), FALSE);

  /*   telephone model/view 
   */
  model = rubrica_model_new(TELEPHONE_MODEL);
  gtk_tree_view_set_model(GTK_TREE_VIEW(form->private->tree), 
			  GTK_TREE_MODEL(model));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(form->private->tree));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE); 

  rubrica_model_add_columns(GTK_TREE_VIEW(form->private->tree),
			    TELEPHONE_COLUMNS);
  
  g_signal_connect(G_OBJECT(selection), "changed", 
		   G_CALLBACK(form_telephone_list_activated), form);
  g_object_unref(model);


  /*    telephone buttons 
  */
  g_signal_connect (G_OBJECT (form->private->add), "clicked", 
		    G_CALLBACK (form_tel_add_clicked), form); 
  g_signal_connect (G_OBJECT (form->private->mod), "clicked", 
		    G_CALLBACK (form_tel_mod_clicked), form); 
  g_signal_connect (G_OBJECT (form->private->del), "clicked", 
		    G_CALLBACK (form_tel_del_clicked), form); 

  
  return form;
}


/**
 * rubrica_form_phone_free
 * @form: a #RubricaFormPhone
 * 
 * free the #RubricaFormPhone
 */
void
rubrica_form_phone_free(RubricaFormPhone* form)
{
  g_return_if_fail(IS_RUBRICA_FORM(form));

  g_object_unref(form);   
}
